#!/bin/sh
#
#        VideoEdge IPSpeedDome Firmware Upgrade
#
#          Copyright 2008 American Dynamics
#

###################### Functions ########################################
v1MigPreserveNewFiles()
{
# migration from V1 -> V2
# remove these old files so that we get the factory defaults
# RD note to self - remember there is a relatively short limit on the max length of a linux command on our busybox implementation
    rm \
 ./etc/lighttpd.conf \
 ./var/spool/cron/crontabs/root \
 ./opt/ad/etc/ad.conf \
 ./opt/ad/etc/conf/CAMBLKSETTINGS_factory.conf \
 ./opt/ad/etc/conf/CONFIGBK.conf \
 ./opt/ad/etc/conf/DIOM.conf \
 ./opt/ad/etc/conf/FLTM.conf \
 ./opt/ad/etc/conf/ENVM.conf \
 ./opt/ad/etc/conf/MDSV.conf \
 ./opt/ad/etc/conf/ONVIF.conf \
 ./opt/ad/etc/conf/PTZP.conf \
 ./opt/ad/etc/conf/RTSP.conf
}


v1MigEditDbFiles()
{
# migration from V1 -> V2
# modify some database tables

    /mnt/flash/usr/bin/sqlite3 ./opt/ad/etc/db/preset.db "\
ALTER TABLE PTZ_PRESET ADD COLUMN ppre_defog_mode INT DEFAULT 0; \
ALTER TABLE PTZ_PRESET ADD COLUMN ppre_ei_mode    INT DEFAULT 1; \
ALTER TABLE PTZ_PRESET ADD COLUMN ppre_awb_state  INT DEFAULT 1; \
CREATE TABLE tempTbl \
( \
  camera_id SMALLINT DEFAULT 0, \
  psqs_id SMALLINT, \
  psqs_step SMALLINT, \
  psqs_dwell SMALLINT, \
  psqs_position SMALLINT \
); \
INSERT INTO tempTbl ( psqs_id, psqs_step, psqs_dwell, psqs_position ) SELECT psqs_id, psqs_step, psqs_dwell, psqs_position FROM PTZ_SEQUENCE_STEP; \
DROP TABLE PTZ_SEQUENCE_STEP; \
ALTER TABLE tempTbl RENAME TO PTZ_SEQUENCE_STEP; \
CREATE TABLE tempTbl2 \
( \
  camera_id SMALLINT DEFAULT 0, \
  pseq_id   SMALLINT, \
  pseq_name VARCHAR, \
  primary key ( camera_id, pseq_id ) \
); \
INSERT INTO tempTbl2 ( pseq_id, pseq_name) SELECT pseq_id, pseq_name FROM PTZ_SEQUENCE; \
DROP TABLE PTZ_SEQUENCE; \
ALTER TABLE tempTbl2 RENAME TO PTZ_SEQUENCE; "

    /mnt/flash/usr/bin/sqlite3 ./opt/ad/etc/db/faultmgr.db "\
DROP TABLE IF EXISTS 'FLTM_FAULT_ACTION'; \
CREATE TABLE 'FLTM_FAULT_ACTION' ( \
  fltm_faid varchar(40) primary key, \
  fltm_fara int, \
  fltm_faca int); "

}

v1MigPreserveDbFiles()
{
# migration from V1 -> V2
# move these old files so that we get the factory defaults
# the original files will be available in /save.1 with an extension of .save

    mv ./opt/ad/etc/db/mediaconfiguration.db ./opt/ad/etc/db/mediaconfiguration.db.save
}


v1EditFiles()
{
# migration from V1 -> V2

    #CAMBLKSETTINGS.conf
    file='./opt/ad/etc/conf/CAMBLKSETTINGS.conf'
    echo '[Defog]'                       >> $file
    echo 'mode=1'                        >> $file
    echo 'Level=1'                       >> $file
    echo ''                              >> $file
    echo '[ElectronicImageStablization]' >> $file
    echo 'mode=1'                        >> $file
    echo ''                              >> $file
    echo '[FrameNoiseReduction]'         >> $file
    echo 'state=2'                       >> $file
    echo ''                              >> $file
    echo '[EnhancedIntensity]'           >> $file
    echo 'mode=1'                        >> $file
    sed -i '/[\DayNightMode]\]/a Level=2' $file
    sed -i 's/stopmax/stop2/g'            $file

    # ADIAPI.conf
    file='./opt/ad/etc/conf/ADIAPI.conf'
    echo 'mode=Standard'   >> $file
    echo 'modeSelected=0'  >> $file
    echo 'strongAdminPW=0' >> $file

    # FTP.conf
    file='./opt/ad/etc/conf/FTP.conf'
    echo ''            >> $file
    echo 'ENABLED=0'   >> $file
    echo 'UPLOADPATH=' >> $file
    echo 'SECUREFTP=0' >> $file

    # IPDOME.conf
    file='./opt/ad/etc/conf/IPDOME.conf'
    sed -i '/ADMIN_EMAIL_ADDRESS/a ADMIN_PW_COMPLEX=0' $file
    sed -i '/ADMIN_EMAIL_ADDRESS/a ENHANCED_SECURITY=' $file
    echo ''                   >> $file
    echo '[TRICKLE_SETTINGS]' >> $file
    echo 'ENABLED=1'          >> $file
    echo 'CAP=50'             >> $file

    # MAIL.conf
    file='./opt/ad/etc/conf/MAIL.conf'
    sed -i '/\[MAIL_SETTINGS\]/a ENABLED=0' $file

    # MEDIAPROFILE.conf
    file='./opt/ad/etc/conf/MEDIAPROFILE.conf'
    sed -i '/\[1\]/a enabled=1' $file
    sed -i '/\[2\]/a enabled=1' $file

    # PTZW.conf
    file='./opt/ad/etc/conf/PTZW.conf'
    sed -i '/[\home]\]/a camera=0'        $file

    # TIME.conf
     file='./opt/ad/etc/conf/TIME.conf'
     sed -i '/Synch/a DTSet=0'            $file


}


fatal()
{
    local result=$?
    echo Error: $* 1>&2
    logger -t ad_installer -p user.err "Error: $*"
    exit $result
}

warning()
{
    echo Warning: $*
}

# YT 20120606: this should now be copy from one patition to other
extract_backups()
{
    local upgrade_tar=$1
    tar -xzvpOf ${upgrade_tar:?undefined} inner.tar.gz | tar zpvxf - -C /  \
      opt/ad/bin/config_backup.sh \
      opt/ad/bin/camerablocksettings \
      opt/ad/bin/config_encrypt.sh \
      opt/ad/etc/conf/CONFIGBK.conf \
      opt/ad/etc/conf/CONFIGPW.conf || fatal "restore of backup scripts failed"
   echo backup scripts restored
}

# YT20120606 for future use
check_md5()
{
# check md5
    echo "extracting inner tarball md5 checksum"
    tar -C /tmp/ -xpvzf $firmware inner.tar.gz.md5 2>&1 || fatal "unable to extract md5 checksum"

    sample1=`tar -xzOvf $firmware inner.tar.gz|md5sum|head -c 32`
    sample2=`cat /tmp/inner.tar.gz.md5|head -c 32`

    echo "checking md5"
    echo "sample1:$sample1 sample2:$sample2"

   if [ $sample1 != $sample2 ]; then
        fatal "md5 checksums do not match"
    fi
    echo "md5 checksums match"
}

back_up()
{
############################################################
# Pre-Upgrade phase (delete old files)
############################################################

# Extract upgrade versions of backup utilities
# No longer necessary as we run from new partition
# extract_backups $firmware
# Ensure camera block settings are stored correctly
camblkrestorerevision=25 # ipdome.1.0.25.xxx.tar.gz
if [ "$currentrevision" -lt "$camblkrestorerevision" -a "$currentmajor" -eq "1" -a "$currentminor" -eq "0" ]; then
    # Remove old CAMBLK*.conf files if present, keeping only CAMBLK_X22.conf, CAMBLK_X35.conf and CAMBLK_X35_N.conf
    #| egrep -v "CAMBLK_X22\.conf|CAMBLK_X35\.conf|CAMBLK_X35_N\.conf"
    # removed match to these files as they were gen 1 only
    find /opt/ad/etc/conf -name "CAMBLK*.conf" | xargs rm -f
fi

# backup configuration
backup_output=`/mnt/flash/opt/ad/bin/config_backup.sh /`
backup_filename=`echo $backup_output | awk '{print $NF}'`
# Amend bootargs if required
#configure_bootargs

}

restore()
{
    if [ `/opt/ad/bin/domeinfo | grep -c 'version 1.0'` -ne 0 ];
    then
        # we are migrating from version 1.x to 2.x !!

        # park original restore files in /save/version.1
        mkdir -p /mnt/flash/save/version.1
        /mnt/flash/opt/ad/bin/config_restore.sh $backup_filename /mnt/flash/save/version.1 /mnt/flash/

        # before we start save a copy of files as they were originally
        backup_filename=${backup_filename%.*}.tgz
        $( cd /mnt/flash/save/version.1; tar -zpcf $backup_filename .  )
        cp $backup_filename /mnt/flash/save

        # make changes
        $( cd /mnt/flash/save/version.1; v1MigPreserveNewFiles; v1MigPreserveDbFiles; v1MigEditDbFiles; v1EditFiles; )

        # make a new backup and apply it to the target file system
        $( cd /mnt/flash/save/version.1; tar -zpcf $backup_filename .  )
        /mnt/flash/opt/ad/bin/config_restore.sh $backup_filename /mnt/flash/ /mnt/flash/

        # dont need version.1 any more
        rm -R /mnt/flash/save/version.1
        echo "Dir listing ....."
        ls -R /mnt/flash/save/

    else
        # config_restore.sh can accept seperate directories for destination and tools
        /mnt/flash/opt/ad/bin/config_restore.sh $backup_filename /mnt/flash/ /mnt/flash/
    fi

    # MDSV.conf V2.1.0 
    # make sure mjpeg stream 3 is enabled by default                                                                                         
    sed -i '/\[JPEG\]/{n;s/server_enabled=0/server_enabled=1/}' /mnt/flash/opt/ad/etc/conf/MDSV.conf              
}

merge_files()
{
	# merge conf files
	if [ -f /mnt/flash/opt/ad/bin/config_merge.sh ] ; then
		/mnt/flash/opt/ad/bin/config_merge.sh
	else
		echo "config_merge.sh file does not exist"
	fi
}

# YT: not sure if this belong here or in factory programming.  Place here for now.
initial_setup()
{
    /mnt/flash/opt/ad/bin/config_factory_create.sh
}

############################################################
# Reads the firmware version from the passed MSP430 firmware file
############################################################
get_file_firmware_version()
{
    local FILE=$1
    local LSB="00"
    local MSB="00"

    # Get the header line from the file
    line=`grep S113D000 $FILE`

    # Get the version low byte
    MSB=`echo $line | cut -c23-24`

    # Get the version hi byte
    LSB=`echo $line | cut -c21-22`

    # return the version
    echo "$MSB$LSB"
}

############################################################
# Check for & if present install new MSP430 firmware
############################################################
check_for_msp430_firmware()
{
    local psu=0
    local iob=0
    local MOUNT_DIR=$1
    local INST_DIR="opt/ad/lib"
    local COPY_DIR="lib/firmware"
    local PSU_FIRMWARE="MSP430_PSU.s19"
    local IOB_FIRMWARE="MSP430_IOB.s19"
    local PSU_DEV_PATH="/sys/bus/i2c/devices/1-0046"
    local IOB_DEV_PATH="/sys/bus/i2c/devices/3-0044"

    # Is there a new PSU firmware file to be installed?
    if [ ! -f "$MOUNT_DIR/$INST_DIR/$PSU_FIRMWARE" ];
    then
        echo "PSU firmware not included in installation package."
    else
        # Copy file to the new
        # Move file to installation directory
        cp $MOUNT_DIR/$INST_DIR/$PSU_FIRMWARE  $MOUNT_DIR/$COPY_DIR/$PSU_FIRMWARE
        mv $MOUNT_DIR/$INST_DIR/$PSU_FIRMWARE /$COPY_DIR/$PSU_FIRMWARE
        if [ $? -ne 0 ];
        then
            echo "Failed to move $MOUNT_DIR/$INST_DIR/$PSU_FIRMWARE to /$COPY_DIR/$PSU_FIRMWARE."
            return
        else
            # Set download flag
            psu=1
        fi
    fi

    # Is there a new IOB firmware file to be installed?
    if [ ! -f "$MOUNT_DIR/$INST_DIR/$IOB_FIRMWARE" ];
    then
        echo "IOB firmware not included in installation package."
    else
        # Move file to installation directory
        cp $MOUNT_DIR/$INST_DIR/$IOB_FIRMWARE  $MOUNT_DIR/$COPY_DIR/$IOB_FIRMWARE
        mv $MOUNT_DIR/$INST_DIR/$IOB_FIRMWARE /$COPY_DIR/$IOB_FIRMWARE
        if [ $? -ne 0 ];
        then
            echo "Failed to move $MOUNT_DIR/$INST_DIR/$IOB_FIRMWARE to /$COPY_DIR/$IOB_FIRMWARE."
            return
        else
            # Set download flag
            iob=1
        fi
    fi

    # If there is nothing to download just return
    if [ $psu -eq 0 ] && [ $iob -eq 0 ];
    then
        return
    fi

    # Try to download the firmware
    #
    # PSU
    if [ $psu -eq 1 ];
    then
        download_psu_msp430 "/$COPY_DIR/$PSU_FIRMWARE" "$PSU_DEV_PATH"
    fi

    # IOB
    if [ $iob -eq 1 ];
    then
        download_iob_msp430 "/$COPY_DIR/$IOB_FIRMWARE" "$IOB_DEV_PATH"
    fi

}

############################################################
# Download firmware to the PSU MSP430
############################################################
download_psu_msp430()
{
    local FIRMWARE=$1
    local DEV_PATH=$2
    local VER_ATTR="$DEV_PATH/msp430"
    local NEW_VER="0000"
    local OLD_VER="0000"
    local MIN_VER="0110"
    local STATUS="XX"
    local DOWNLOAD_ATTR="$DEV_PATH/download"
    local STATUS_ATTR="$DEV_PATH/status"
    local count=0


    # Check to see if firmware file exists
    if [ ! -f "$FIRMWARE" ];
    then
        echo "Missing firmware file: $FIRMWARE"
        return
    fi


    # Make sure the driver's attributes exists
    if [ ! -e "$STATUS_ATTR" ];
    then
        echo "Missing device driver attributes: $STATUS_ATTR"
        return
    fi


    # Get the current firmware version
    OLD_VER=`cat $VER_ATTR`

    # Get the new firmware version
    NEW_VER=`get_file_firmware_version $FIRMWARE`

    # Make sure the current version is upgradable
    if [ "$OLD_VER" -lt "$MIN_VER" ]
    then
        echo "The current PSU firmware can not be upgraded automatically."
        rm -f $FIRMWARE
        return
    fi

    # Make sure the new version is a newer version than what's already running
    if [ "$NEW_VER" -lt "$OLD_VER" ]
    then
        echo "The current PSU firmware(ver:$OLD_VER) is newer than the version in the installation package(ver:$NEW_VER), ignoring."
        rm -f $FIRMWARE
        return
    fi

    # Make sure they are different versions
    if [ "$OLD_VER" == "$NEW_VER" ];
    then
        echo "The new PSU firmware is the same version as the current firmware(ver:$OLD_VER), ignoring."
          rm -f $FIRMWARE
        return
    fi

    # Start the download, it does NOT block
    echo "Starting PSU firmware download."
    echo 0 > $DOWNLOAD_ATTR
    echo 1 > $DOWNLOAD_ATTR

    # Wait for the download to complete, wait a max of 40 sec
    sleep 2
    STATUS=`cat $DOWNLOAD_ATTR`
    while [ "$STATUS" == "31" ] && [ $count -lt 20 ]
    do
        # Get download status
        STATUS=`cat $DOWNLOAD_ATTR`
        echo "Status = $STATUS"
        let count=$count+1
        sleep 2
    done

    # Check the download result, 30 indicates ok
    STATUS=`cat $DOWNLOAD_ATTR`
    if [ "$STATUS" == "30" ]
    then
        # Check for successful restart of the MSP430, wait a max of 20 sec
        STATUS="XX"
        count=0
        while [ "$STATUS" != "ST" ] && [ $count -lt 10 ]
        do
            # Get status
            STATUS=`cat $STATUS_ATTR`
            echo "Status = $STATUS"
            let count=$count+1
            sleep 2
        done
    else
        echo "*** PSU Download failed! ****"
        return 1
    fi

    # Did it restart?
    if [ "$STATUS" != "ST" ];
    then
        echo "MSP430 restart failed."
        return
    fi

    # Get the new firmware version
    NEW_VER=`cat $VER_ATTR`
    echo "Previous version:$OLD_VER. Current version:$NEW_VER"
    echo "PSU download complete."

    # Remove firmware image file
    rm -f $FIRMWARE
}

############################################################
# Download firmware to the POE/ENV MSP430
############################################################
download_iob_msp430()
{
    local FIRMWARE=$1
    local DEV_PATH=$2
    local VERSION_ATTR="$DEV_PATH/MSP430_version"
    local NEW_VER="0000"
    local OLD_VER="0000"
    local MIN_VER="0115"
    local STATUS="XX"
    local DOWNLOAD_ATTR="$DEV_PATH/download"
    local TYPE_ATTR="$DEV_PATH/io_type"
    local count=0


    # Check to see if firmware file exists
    if [ ! -f "$FIRMWARE" ];
    then
        echo "Missing firmware file: $FIRMWARE"
        return
    fi


    # Make sure the driver's attributes exists
    if [ ! -e "$DOWNLOAD_ATTR" ];
    then
        echo "Missing device driver attributes: $DOWNLOAD_ATTR"
        return
    fi

    if [ ! -e "$TYPE_ATTR" ];
    then
        echo "Missing device driver attributes: $TYPE_ATTR"
        return
    fi

    if [ ! -e "$VERSION_ATTR" ];
    then
        echo "Missing device driver attributes: $VERSION_ATTR"
        return
    fi


    # Get the current firmware version
    OLD_VER=`cat $VERSION_ATTR`

    # Get the new firmware version
    NEW_VER=`get_file_firmware_version $FIRMWARE`

    # Make sure the current version is upgradable
    if [ "$OLD_VER" -lt "$MIN_VER" ]
    then
        echo "The current IOB firmware can not be upgraded automatically."
        rm -f $FIRMWARE
        return
    fi

    # Make sure the new version is a newer version than what's already running
    if [ "$NEW_VER" -lt "$OLD_VER" ]
    then
        echo "The current IOB firmware(ver:$OLD_VER) is newer than the version in the installation package(ver:$NEW_VER), ignoring."
        rm -f $FIRMWARE
        return
    fi

    # Make sure they are different versions
    if [ "$OLD_VER" == "$NEW_VER" ];
    then
        echo "The new IOB firmware is the same version as the current firmware(ver:$OLD_VER), ignoring."
           rm -f $FIRMWARE
        return
    fi

    # Start the download, it does NOT block
    echo "Starting IOB firmware download."
    echo 0 > $DOWNLOAD_ATTR
    echo 1 > $DOWNLOAD_ATTR

    # Wait for the download to complete, wait a max of 40 sec
    sleep 2
    STATUS=`cat $DOWNLOAD_ATTR`
    while [ "$STATUS" == "31" ] && [ $count -lt 20 ]
    do
        # Get download status
        STATUS=`cat $DOWNLOAD_ATTR`
        echo "Status = $STATUS"
        let count=$count+1
        sleep 2
    done

    # Check the download result, 30 indicates ok
    STATUS=`cat $DOWNLOAD_ATTR`
    if [ "$STATUS" == "35" ]
    then
        # Check for successful restart of the MSP430, wait a max of 20 sec
        # Get device type
        STATUS=`cat $TYPE_ATTR`
        count=0
        while [ "$STATUS" == "2" ] && [ $count -lt 10 ]
        do
            # Get status
            STATUS=`cat $TYPE_ATTR`
            echo "Status = $STATUS"
            let count=$count+1
            sleep 2
        done
    else
        echo "*** IOB Download failed! ****"
        return 1
    fi

    # Did it restart?
    if [ "$STATUS" == "2" ];
    then
        echo "MSP430 restart failed."
        return
    fi

    # Get the new firmware version
    NEW_VER=`cat $VERSION_ATTR`
    echo "Previous version:$OLD_VER. Current version:$NEW_VER"
    echo "IOB download complete."
    sleep 3

    # Remove firmware image file
    rm -f $FIRMWARE
}



##################### Begin Script Operation ################
exec 200>/tmp/i625ptz_installer.lock
if ! flock -n 200  ; then
   echo "another instance is running";
   logger -t ad_installer -p user.err "Error: installer already running"
   exit 1
fi

# Direct output from remainder of this script
exec 1>/var/log/update.log 2>&1

echo "Firmware upgrade started at:"
date

if [ 1 -ne $# ]; then
    echo Usage: $0 "<firmware filename>"
    exit 1
fi

firmware=$1
if [ ! -f $firmware ]; then
    fatal "firmware file $firmware doesn't exist"
fi

# Detect which bank is currently active
if [ `fw_printenv activeimage` == "activeimage=A" ] ;
then
  echo Active Image is A
  NEW_KERNEL_MTD=/dev/mtd6
  NEW_FS_MTD=/dev/mtd7
  NEW_MTD_NO=7
  NEW_FS_MTDBLOCK=/dev/mtdblock7
else if [ `fw_printenv activeimage` == "activeimage=B" ] ;
then
  echo Active Image is B
  NEW_KERNEL_MTD=/dev/mtd4
  NEW_FS_MTD=/dev/mtd5
  NEW_MTD_NO=5
  NEW_FS_MTDBLOCK=/dev/mtdblock5
else
  fatal "unable to determine current active image"
fi
fi

# output current firwmare version for information purpose
currentfirmware=`/opt/ad/bin/domeinfo | awk '{print$6}'`

# check if we should install ubifs
ubifs=0

if [ `/opt/ad/bin/domeinfo | grep -c "illustra 825"` -eq 1 ] ;
then
  ubifs=1
fi


#extract the kernel
#tar zpxf ${firmware} uImage                   || fatal "extracting uImage"
tar zpxf ${firmware} fw.bin --to-stdout | /opt/ad/bin/config_decrypt.sh | tar zpxf - uImage || fatal "extracting uImage"

if [ $ubifs -eq 1 ] ;
then

  ############### UBI Install #####################
  flash_erase ${NEW_KERNEL_MTD} 0 0                 || fatal "erasing flash partition ${NEW_KERNEL_MTD}"
  nandwrite -p ${NEW_KERNEL_MTD} uImage             || fatal "writing new kernel"

  ubiformat -y ${NEW_FS_MTD} -s 512 -O 2048         || fatal "formatting flash partition ${NEW_FS_MTD}"
  mkdir -p /mnt/flash                               || fatal "creating directory /mnt/flash"
  ubiattach /dev/ubi_ctrl -m ${NEW_MTD_NO} -O 2048  || fatal "ubiattach partition ${NEW_MTD_NO}"

  UBICOUNT=0
  UBISTATUS=0
  UBI1DEV=/dev/ubi1

  echo "Checking ubinfo.."

  # Check that ubiattach has completed before running ubimkvol.
  while [ $UBISTATUS -eq 0 ] && [ $UBICOUNT -lt 10 ]
  do
    ubinfo -d 1

    # the 'ubinfo -d 1' command exits with code 0 if ubi1 device is present.
    if [ $? -eq 0 ] ;
    then
        # also check the ubi1 device file has been created.
        if [ -e $UBI1DEV ] ;
        then
            UBISTATUS=1
        else
            echo "File $UBI1DEV not present"
        fi
    fi

    let UBICOUNT=$UBICOUNT+1
    sleep 1
  done

  if [ $UBISTATUS -eq 0 ] ;
  then
    fatal "ubiattach failed to complete"
  fi

  ubimkvol /dev/ubi1 -N rootfs -s 90MiB             || fatal "creating ubi volume"
  mount -t ubifs -o sync ubi1:rootfs /mnt/flash     || fatal "mounting second filesystem"

else

  ############### JFFS2 Install #####################
  flash_erase -j ${NEW_KERNEL_MTD} 0 0          || fatal "erasing flash partition ${NEW_KERNEL_MTD}"
  nandwrite -p ${NEW_KERNEL_MTD} uImage         || fatal "writing new kernel"

  flash_erase -j ${NEW_FS_MTD} 0 0              || fatal "erasing flash partition ${NEW_FS_MTD}"
  mkdir -p /mnt/flash                           || fatal "creating directory /mnt/flash"
  mount -t jffs2 ${NEW_FS_MTDBLOCK} /mnt/flash  || fatal "mounting second filesystem"

fi

############### Explode the new filesystem image #####################
cd /mnt/flash                                 || fatal "Unable to access /mnt/flash"
#tar zpxf ${firmware} fs --strip-components=1  || fatal "writing new filesystem "
tar zpxf ${firmware} fw.bin --to-stdout | /opt/ad/bin/config_decrypt.sh | tar zpxf - fs --strip-components=1 || fatal "writing new filesystem "

############### initialise some device specific items###############
initial_setup

# backup files from configuration list
back_up

# restore the same files
restore

# merge/add new file changes with cooresponding files
merge_files

############### Update MSP430 firmware if needed #####################
check_for_msp430_firmware /mnt/flash

############### Unmount 'new' file system image #####################
cd /tmp
sync
sync
umount /mnt/flash
rmdir /mnt/flash

############### Swap active image on next boot & Reboot #####################
fw_setenv swap_image 1                        || fatal "unable to swap images"
echo Rebooting now
reboot


